#include "stdafx.h"
#include "DynamicApi.h"
#include "misc.h"
#include "crtdbg.h"

#define GetEachFunc(x,y,z)	if(x==NULL)x=(y)GetFuncAddress(z)
#define GetEachFunc2(x,y)	if(p##x==NULL)p##x=(y)GetFuncAddress(#x)
#define GetEachFunc3(x)		if(p##x==NULL)p##x=(DLL##x)GetFuncAddress(#x)
#define InvokeFunc(x,y,...)	p##x?p##x(__VA_ARGS__):y

User32Api user32;
Shell32Api shell32;
Imm32Api imm32;
WtsApi32Api wtsapi32;



DynamicApi::DynamicApi( const TCHAR *name, BOOL notLoad, DLL_DIR dir, const TCHAR *path )
	: dllHandle( NULL )
	, notFreeLib( notLoad )
{
	switch( dir )
	{
	case SYSTEM32:
		_ASSERT( name != NULL );
		GetSystemDirectory( dllPath, _countof(dllPath ) );
		_tcscat_s( dllPath, _countof(dllPath), _T("\\") );
		break;

	case EXE:
		_ASSERT( name != NULL );
		GetModuleFileName( NULL, dllPath, _countof(dllPath) );
		*( _tcsrchr( dllPath, _T('\\') ) + 1 ) = 0;
		break;

	case CURRENT:
		_ASSERT( name != NULL );
		GetCurrentDirectory( _countof(dllPath ), dllPath );
		_tcscat_s( dllPath, _countof(dllPath), _T("\\") );
		break;

	case SPECIFY:
		_ASSERT( name != NULL );
		_ASSERT( path != NULL );
		if ( path ) {
			_tcscpy_s( dllPath, _countof(dllPath), path );
			if ( dllPath[ _tcslen(dllPath) - 1 ] != _T('\\') ) {
				_tcscat_s( dllPath, _countof(dllPath), _T("\\") );
			}
		}
		else {
			GetSystemDirectory( dllPath, _countof(dllPath ) );
			_tcscat_s( dllPath, _countof(dllPath), _T("\\") );
		}
		break;

	case SEARCH:
		_ASSERT( name != NULL );
		dllPath[0] = 0;
		break;

	case AUTO3264:
		_ASSERT( name == NULL );
		GetModuleFileName( NULL, dllPath, _countof(dllPath) );
		{
			TCHAR *p = _tcsrchr( dllPath, _T('\\') ) + 1;
			TCHAR *e = _tcschr( p, _T('.') );
			if ( e ) *e = 0;
			if ( ( _tcslen( p ) > 2 ) && ( _tcscmp( ( e = p + _tcslen(p) - 2 ), _T("64") ) == 0 ) ) {
					*e = 0;
					_tcscat_s( dllPath, _countof(dllPath), _T("Dll64") );
			}
			else {
				_tcscat_s( dllPath, _countof(dllPath), _T("Dll") );
			}
			_tcscat_s( dllPath, _countof(dllPath), _T(".dll") );
		}
		break;
	}
	if ( name ) _tcscat_s( dllPath, _countof(dllPath), name );
}

DynamicApi::~DynamicApi(void)
{
	if ( !notFreeLib ) {
		FreeLibrary( dllHandle );
		dllHandle = NULL;
	}
}

BOOL DynamicApi::Init(void)
{
	if ( dllHandle == NULL ) {
		if ( notFreeLib ) {
			dllHandle = GetModuleHandle( dllPath );
			if ( dllHandle == NULL ) {
				notFreeLib = FALSE;
				dllHandle = LoadLibrary( dllPath );
			}
		}
		else {
			dllHandle = LoadLibrary( dllPath );
		}
	}

	if ( _tcschr( dllPath, _T('\\') ) == NULL ) {
		GetModuleFileName( dllHandle, dllPath, _countof(dllPath) );
	}

	return dllHandle ? TRUE : FALSE;
}

FARPROC DynamicApi::GetFuncAddress( const char *func )
{
	if ( dllHandle == NULL ) Init();
	return GetProcAddress( dllHandle, func );
}



BOOL User32Api::ChangeWindowMessageFilter( UINT message, DWORD dwFlag )
{
	GetEachFunc3( ChangeWindowMessageFilter );
	return InvokeFunc( ChangeWindowMessageFilter, FALSE, message, dwFlag );
}

BOOL User32Api::SetLayeredWindowAttributes( HWND hwnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags )
{
	GetEachFunc3( SetLayeredWindowAttributes );
	return InvokeFunc( SetLayeredWindowAttributes, FALSE, hwnd, crKey, bAlpha, dwFlags );
}

HDESK User32Api::OpenInputDesktop( DWORD dwFlags, BOOL fInherit, ACCESS_MASK dwDesiredAccess )
{
	GetEachFunc3( OpenInputDesktop );
	return InvokeFunc( OpenInputDesktop, NULL, dwFlags, fInherit, dwDesiredAccess );
}

BOOL User32Api::CloseDesktop( HDESK hDesktop )
{
	GetEachFunc3( CloseDesktop );
	return InvokeFunc( CloseDesktop, FALSE, hDesktop );
}



Shell32Api::Shell32Api(void)
	: DynamicApi( _T("shell32.dll") )
	, pDllGetVersion( NULL )
	, oldVersion( FALSE )
{
	if ( GetSimpleOSVersion() < OS_2000 ) oldVersion = TRUE;
}

HRESULT Shell32Api::DllGetVersion( DLLVERSIONINFO *pdvi )
{
	GetEachFunc2( DllGetVersion, DLLGETVERSIONPROC );
	return InvokeFunc( DllGetVersion, E_NOTIMPL, pdvi );
}



BOOL Imm32Api::ImmDisableIME( DWORD idThread )
{
	GetEachFunc3( ImmDisableIME );
	return InvokeFunc( ImmDisableIME, FALSE, idThread );
}



BOOL WtsApi32Api::WTSRegisterSessionNotification( HWND hwnd, DWORD dwFlags )
{
	GetEachFunc3( WTSRegisterSessionNotification );
	return InvokeFunc( WTSRegisterSessionNotification, FALSE, hwnd, dwFlags );
}

BOOL WtsApi32Api::WTSUnRegisterSessionNotification( HWND hwnd )
{
	GetEachFunc3( WTSUnRegisterSessionNotification );
	return InvokeFunc( WTSUnRegisterSessionNotification, FALSE, hwnd );
}



int QGraphicsTextApi::Render( const LOGFONT& logfont, Gdiplus::GraphicsPath& path, LPCTSTR str, Gdiplus::PointF pntOrigin, bool bFlipY )
{
	GetEachFunc3( Render );
	return InvokeFunc( Render, -1, logfont, path, str, pntOrigin, bFlipY );
}

HANDLE QGraphicsTextApi::New( void )
{
	GetEachFunc3( New );
	return InvokeFunc( New, NULL );
}

void QGraphicsTextApi::SetFont( HANDLE ctx, const LOGFONT& logfont, bool bFlipY )
{
	GetEachFunc3( SetFont );
	InvokeFunc( SetFont, NULL, ctx, logfont, bFlipY );
}

int QGraphicsTextApi::GetGraphicsText( HANDLE ctx, Gdiplus::GraphicsPath& path, LPCTSTR str, Gdiplus::PointF pntOrigin )
{
	GetEachFunc3( GetGraphicsText );
	return InvokeFunc( GetGraphicsText, -1, ctx, path, str, pntOrigin );
}

void QGraphicsTextApi::Delete( HANDLE ctx )
{
	GetEachFunc3( Delete );
	InvokeFunc( Delete, NULL, ctx );
}
