#ifndef _DYNAMIC_API_H_
#define _DYNAMIC_API_H_

#include <windows.h>
#include <shlwapi.h>
#include <psapi.h>
#include <gdiplus.h>


class DynamicApi
{
public:
	typedef enum _DLL_DIR {
		SYSTEM32,
		EXE,
		CURRENT,
		SPECIFY,
		SEARCH,
		AUTO3264
	} DLL_DIR;

	DynamicApi( const TCHAR *name, BOOL notLoad = FALSE, DLL_DIR dir = SYSTEM32, const TCHAR *path = NULL );
	virtual ~DynamicApi(void);
	BOOL Init();
	const TCHAR *GetPath(void) { return dllPath; };

protected:
	HMODULE dllHandle;
	BOOL notFreeLib;
	TCHAR dllPath[_MAX_PATH];

	FARPROC GetFuncAddress( const char *func );
};



class User32Api : public DynamicApi
{
public:
	User32Api(void)
		: DynamicApi( _T("user32.dll"), TRUE )
		, pChangeWindowMessageFilter( NULL )
		, pSetLayeredWindowAttributes( NULL )
		, pOpenInputDesktop( NULL )
		, pCloseDesktop( NULL )
	{};
	virtual ~User32Api(void) {};
	BOOL ChangeWindowMessageFilter( UINT message, DWORD dwFlag );
	BOOL SetLayeredWindowAttributes( HWND hwnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags );
	HDESK OpenInputDesktop( DWORD dwFlags, BOOL fInherit, ACCESS_MASK dwDesiredAccess );
	BOOL CloseDesktop( HDESK hDesktop );

private:
	typedef BOOL(WINAPI *DLLChangeWindowMessageFilter)( UINT, DWORD );
	typedef BOOL(WINAPI *DLLSetLayeredWindowAttributes)( HWND hwnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags );
	typedef HDESK(WINAPI *DLLOpenInputDesktop)( DWORD, BOOL, ACCESS_MASK );
	typedef BOOL(WINAPI *DLLCloseDesktop)( HDESK );
	DLLChangeWindowMessageFilter pChangeWindowMessageFilter;
	DLLSetLayeredWindowAttributes pSetLayeredWindowAttributes;
	DLLOpenInputDesktop pOpenInputDesktop;
	DLLCloseDesktop pCloseDesktop;
};



class Shell32Api : public DynamicApi
{
public:
	Shell32Api(void);
	virtual ~Shell32Api(void) {};
	HRESULT DllGetVersion( DLLVERSIONINFO *pdvi );
	BOOL IsOldVersion(void) { return oldVersion; };

private:
	DLLGETVERSIONPROC pDllGetVersion;
	BOOL oldVersion;
};



class Imm32Api : public DynamicApi
{
public:
	Imm32Api(void)
		: DynamicApi( _T("imm32.dll") )
		, pImmDisableIME( NULL )
	{};
	virtual ~Imm32Api(void) {};
	BOOL ImmDisableIME( DWORD idThread );

private:
	typedef BOOL(WINAPI *DLLImmDisableIME)( DWORD );
	DLLImmDisableIME pImmDisableIME;
};



class WtsApi32Api : public DynamicApi
{
public:
	WtsApi32Api(void)
		: DynamicApi( _T("wtsapi32.dll") )
		, pWTSRegisterSessionNotification( NULL )
		, pWTSUnRegisterSessionNotification( NULL )
	{};
	virtual ~WtsApi32Api(void) {};
	BOOL WTSRegisterSessionNotification( HWND hwnd, DWORD dwFlags );
	BOOL WTSUnRegisterSessionNotification( HWND hwnd );

private:
	typedef BOOL(WINAPI *DLLWTSRegisterSessionNotification)( HWND, DWORD );
	typedef BOOL(WINAPI *DLLWTSUnRegisterSessionNotification)( HWND );
	DLLWTSRegisterSessionNotification pWTSRegisterSessionNotification;
	DLLWTSUnRegisterSessionNotification pWTSUnRegisterSessionNotification;
};



class QGraphicsTextApi : public DynamicApi
{
public:
	QGraphicsTextApi(void)
		: DynamicApi( _T("QGraphicsText.dll"), FALSE, EXE )
		, pRender( NULL )
		, pNew( NULL )
		, pSetFont( NULL )
		, pGetGraphicsText( NULL )
		, pDelete( NULL )
	{};
	virtual ~QGraphicsTextApi(void) {};
	int Render( const LOGFONT& logfont, Gdiplus::GraphicsPath& path, LPCTSTR str, Gdiplus::PointF pntOrigin, bool bFlipY = true );

	HANDLE New( void );
	void SetFont( HANDLE ctx, const LOGFONT& logfont, bool bFlipY = true );
	int GetGraphicsText( HANDLE ctx, Gdiplus::GraphicsPath& path, LPCTSTR str, Gdiplus::PointF pntOrigin );
	void Delete( HANDLE ctx );

private:
	typedef int (*DLLRender)( const LOGFONT& logfont, Gdiplus::GraphicsPath& path, LPCTSTR str, Gdiplus::PointF pntOrigin, bool bFlipY );
	typedef HANDLE (*DLLNew)( void );
	typedef void (*DLLSetFont)( HANDLE ctx, const LOGFONT& logfont, bool bFlipY );
	typedef int (*DLLGetGraphicsText)( HANDLE ctx, Gdiplus::GraphicsPath& path, LPCTSTR str, Gdiplus::PointF pntOrigin );
	typedef void (*DLLDelete)( HANDLE ctx );
	DLLRender pRender;
	DLLNew pNew;
	DLLSetFont pSetFont;
	DLLGetGraphicsText pGetGraphicsText;
	DLLDelete pDelete;
};



extern User32Api user32;
extern Shell32Api shell32;
extern Imm32Api imm32;
extern WtsApi32Api wtsapi32;

#endif//_DYNAMIC_API_H_
