#include "StdAfx.h"
#include "GnyaFaceGdi.h"


GnyaFaceGdi::GnyaFaceGdi( AppProfile &ref )
	: prof( ref )
	, handPen( NULL )
	, dotPen( NULL )
	, dotBrush( NULL )
	, dayFont( NULL )
{
}

GnyaFaceGdi::~GnyaFaceGdi(void)
{
	if ( handPen ) DeleteObject( handPen );
	if ( dotBrush ) DeleteObject( dotBrush );
	if ( dotPen ) DeleteObject( dotPen );
	if ( dayFont ) DeleteObject ( dayFont );
}

void GnyaFaceGdi::SetX( void *p, int idx, double value )
{
	POINT *outp = (POINT*)p;
	outp[idx].x = (LONG)value;
}

void GnyaFaceGdi::SetY( void *p, int idx, double value )
{
	POINT *outp = (POINT*)p;
	outp[idx].y = (LONG)value;
}

double GnyaFaceGdi::GetX( void *p, int idx )
{
	POINT *inp = (POINT*)p;
	return (double)inp[idx].x;
}

double GnyaFaceGdi::GetY( void *p, int idx )
{
	POINT *inp = (POINT*)p;
	return (double)inp[idx].y;
}

void GnyaFaceGdi::SetRect( void *p, double x, double y, double width, double height )
{
	RECT *outr = (RECT*)p;
	outr->left = (LONG)x;
	outr->top = (LONG)y;
	outr->right = (LONG)( x + width + 0.5 );
	outr->bottom = (LONG)( y + height + 0.5 );
}

void GnyaFaceGdi::ShiftRect( RECT *inRect, RECT *outRect, int hShift, int vShift )
{
	outRect->left = inRect->left + hShift;
	outRect->top = inRect->top + vShift;
	outRect->right = inRect->right + hShift;
	outRect->bottom = inRect->bottom + vShift;
}

void GnyaFaceGdi::UpdateSize( int width, int height, void *param )
{
	dots.clear();

	RECT rp;
	for ( int h = 0; h < 12; h++ ) {
		if ( GetClockDot( h, prof.get_clock_type(), width, height, &rp ) ) {
			dots.push_back( rp );
		}
	}

	if ( handPen ) DeleteObject( handPen );
	handPen = CreatePen( PS_SOLID, ( ( ( width + height ) / 80 + 2 ) * prof.get_hand_width() / 100 ), prof.get_color_hand() );

	LOGFONT scaledLf = prof.get_lf();
	scaledLf.lfHeight = (LONG)( (double)scaledLf.lfHeight * (double)height / (double)prof.get_heightForFont() );

	if ( dayFont ) DeleteObject ( dayFont );
	dayFont = CreateFontIndirect( &scaledLf );
}

void GnyaFaceGdi::UpdateSettings(void)
{
	if ( dotBrush ) DeleteObject( dotBrush );
	dotBrush = CreateSolidBrush( prof.get_color_dot() );
	if ( dotPen ) DeleteObject( dotPen );
	dotPen = CreatePen( PS_SOLID, 0, prof.get_color_dot() );
}

void GnyaFaceGdi::Draw( int width, int height, int tm, int day, void *param )
{
	HDC memDC = (HDC)param;

	POINT gxp[4];
	HPEN prePen;
	HBRUSH preBrush;
	HFONT preFont;

	prePen = (HPEN)SelectObject( memDC, handPen );

	GetBezierPoints( tm, width, height, gxp, !prof.get_friendlyGnya() );
	int bezierDivide = prof.get_bezierDivide();
	POINT *pp = new POINT[ bezierDivide + 1 ];

	int start, count;
	GetBezierCurve( bezierDivide, gxp, pp, prof.get_friendlyGnya() ? true : false, width, height, &start, &count );
	if ( prof.get_friendlyGnya() ) {
		Polyline( memDC, &(pp[ start ]), count );
	}
	else {
		Polyline( memDC, pp, bezierDivide + 1 );
	}
	delete[] pp;

	preBrush = (HBRUSH)SelectObject( memDC, dotBrush );
	SelectObject( memDC, dotPen );
	for ( std::vector<RECT>::iterator iter = dots.begin(); iter != dots.end(); iter++ ) {
		Ellipse( memDC, (*iter).left, (*iter).top, (*iter).right, (*iter).bottom );
	}
	SelectObject( memDC, preBrush );
	SelectObject( memDC, prePen );

	if ( prof.get_drawDay() ) {

		RECT txtRect, backRect;
		txtRect.left = 0;
		txtRect.top = height*2/3;
		txtRect.right = width;
		txtRect.bottom = height;

		SetBkColor( memDC, prof.get_color_dial() );
		SetBkMode( memDC, TRANSPARENT );

		preFont = (HFONT)SelectObject( memDC, dayFont );

		TCHAR strTemp[256];
		_stprintf_s( strTemp, _countof(strTemp), _T("%d"), day );

		SetTextColor( memDC, prof.get_color_dial() );
		ShiftRect( &txtRect, &backRect, -1, 0 );
		DrawText( memDC, strTemp, (int)_tcslen(strTemp), &backRect, DT_CENTER | DT_TOP );
		ShiftRect( &txtRect, &backRect, 1, 0 );
		DrawText( memDC, strTemp, (int)_tcslen(strTemp), &backRect, DT_CENTER | DT_TOP );
		ShiftRect( &txtRect, &backRect, 0, -1 );
		DrawText( memDC, strTemp, (int)_tcslen(strTemp), &backRect, DT_CENTER | DT_TOP );
		ShiftRect( &txtRect, &backRect, 0, 1 );
		DrawText( memDC, strTemp, (int)_tcslen(strTemp), &backRect, DT_CENTER | DT_TOP );

		SetTextColor( memDC, prof.get_color_day() );
		DrawText( memDC, strTemp, (int)_tcslen(strTemp), &txtRect, DT_CENTER | DT_TOP );

		SelectObject( memDC, preFont );
	}
}
