#include "stdafx.h"
#include <windows.h>
#include <shlwapi.h>
#include "misc.h"


SIMPLE_OSVERSION GetSimpleOSVersion( void )
{
	OSVERSIONINFOEX ver;
	static SIMPLE_OSVERSION soVer = OS_UNKNOWN;

	if ( soVer != OS_UNKNOWN ) return soVer;

	ver.dwOSVersionInfoSize = sizeof(ver);
	GetVersionEx( (LPOSVERSIONINFO)&ver );

	switch( ver.dwPlatformId )
	{
	case VER_PLATFORM_WIN32s:
		soVer = OS_OLD;
		break;

	case VER_PLATFORM_WIN32_WINDOWS:
		soVer = OS_9X;
		break;

	case VER_PLATFORM_WIN32_NT:
		switch( ver.dwMajorVersion )
		{
		case 3:
			soVer = OS_OLD;
			break;

		case 4:
			soVer = OS_NT4;
			break;

		case 5:
			switch( ver.dwMinorVersion )
			{
			case 0:
				soVer = OS_2000;
				break;

			case 1:
				soVer = OS_XP;
				break;

			case 2:
				switch( ver.wProductType )
				{
				case VER_NT_WORKSTATION:
					soVer = OS_XP;
					break;

				default:
					soVer = OS_S2003;
					break;
				}
				break;

			default:
				soVer = OS_NEWER;
				break;
			}
			break;

		case 6:
			switch( ver.dwMinorVersion )
			{
			case 0:
				switch( ver.wProductType )
				{
				case VER_NT_WORKSTATION:
					soVer = OS_VISTA;
					break;

				default:
					soVer = OS_S2008;
					break;
				}
				break;

			case 1:
				switch( ver.wProductType )
				{
				case VER_NT_WORKSTATION:
					soVer = OS_7;
					break;

				default:
					soVer = OS_S2008R2;
					break;
				}
				break;

			default:
				soVer = OS_NEWER;
				break;
			}
			break;

		default:
			soVer = OS_NEWER;
			break;
		}
		break;
	}

	return soVer;
}

ULONGLONG GetLocalTime64( void )
{
	FILETIME ft;
	ULARGE_INTEGER ui;
	SYSTEMTIME st;

	GetLocalTime( &st );
	SystemTimeToFileTime( &st, &ft );

	ui.LowPart = ft.dwLowDateTime;
	ui.HighPart = ft.dwHighDateTime;

	return ui.QuadPart;
}

ULONGLONG AltGetTickCount64(void)
{
	static ULONGLONG bootTime = 0;

	typedef ULONGLONG (WINAPI* DLLGetTickCount64)(void);
	static DLLGetTickCount64 pGetTickCount64 = NULL;
	static BOOL getProcFailed = FALSE;

	if ( ( GetSimpleOSVersion() >= OS_VISTA ) && !getProcFailed ) {
		if ( pGetTickCount64 == NULL ) {
			pGetTickCount64 = (DLLGetTickCount64)GetProcAddress( GetModuleHandle( _T("kernel32.dll") ), "GetTickCount64" );
			if ( pGetTickCount64 == NULL ) getProcFailed = TRUE;
		}
		if ( pGetTickCount64 ) return pGetTickCount64();
	}

	if ( bootTime == 0 ) {
		bootTime = GetLocalTime64() / 10000 - (ULONGLONG)GetTickCount();
	}

	return GetLocalTime64() / 10000 - bootTime;
}
